#!/usr/bin/python3
#
#   Written by Christian D. Powell, 10/14/2019
#   Copyright (c) 2019. Christian D. Powell and Hunter N.B. Moseley
#   All rights reserved.
#
"""

"""
import jsonpickle
from ProtamineAnalysis import *
from scipy.stats import ttest_ind

__version__ = '1.0.0'


if __name__ == '__main__':

    """
    Open Datasets
    """
    with open('Docs/Datasets/eutherian_p1.json', 'r') as f:
        ep1_align = jsonpickle.decode(f.read())
    with open('Docs/Datasets/eutherian_p2.json', 'r') as f:
        ep2_align = jsonpickle.decode(f.read())
    with open('Docs/Datasets/metatherian.json', 'r') as f:
        mp1_align = jsonpickle.decode(f.read())
    with open('Docs/Datasets/fish.json', 'r') as f:
        fish_align = jsonpickle.decode(f.read())

    alignments = [ep1_align, ep2_align, ep2_align, mp1_align, fish_align]
    alignment_names = ['Eutherian P1 Sperm Protamines',
                       'Eutherian P2 Sperm Protamines',
                       'Truncated Eutherian P2 Sperm Protamines',
                       'Metatherian P1 Sperm Protamines',
                       'Fish Protamines']

    """
    Priniting
    """
    print("#####################################")
    print("#            Alignments             #")
    print("#                                   #")
    print("# Conserved positions marked with * #")
    print("#####################################")
    ep1_ids = ['HSP1_HUMAN', 'HSP1_BOVIN', 'HSP1_HORSE', 'HSP1_MOUSE', 'HSP1_RAT', 'HSP1_ORCOR', 'HSP1_PIG',
               'HSP1_GORGO', 'HSP1_CAPHI', 'HSP1_SHEEP']
    mp1_ids = ['HSP1_MACRU', 'HSP1_MONDO', 'HSP1_DIDVI', 'HSP1_MACGI', 'HSP1_WALBI', 'HSP1_PHACI', 'HSP1_PLAIN',
               'HSP1_PLAMS', 'HSP1_THYCY', 'HSP1_PSECU']
    print("Eutherian P1 Sperm Protamines")
    ep1_align.print_alignment_2(line_len=70, ids=ep1_ids)
    print("Eutherian P2 Sperm Protamines")
    ep2_align.print_alignment_2(line_len=70)
    print('Truncated Eutherian P2 Sperm Protamines')
    ep2_align.print_alignment_2(line_len=70, start=48)
    print('Metatherian P1 Sperm Protamines')
    mp1_align.print_alignment_2(line_len=70, ids=mp1_ids)
    print('Fish Protamines')
    fish_align.print_alignment_2(line_len=70)

    # Supplemental tables
    print("##############################")
    print("# Conserved Residue Rankings #")
    print("##############################")
    print()
    for x in range(4):
        print(alignment_names[x])
        if x == 2:
            analysis.ranking_table(alignments[x], start=48)
        else:
            analysis.ranking_table(alignments[x])

    """
    Frequency Analyses
    """
    # whole sequence charged residue frequencies
    ep1_charge_freqs = [analysis.frequency(pro, ['R', 'K']) for pro in ep1_align.protein_list]
    ep2_charge_freqs = [analysis.frequency(pro, ['R', 'K']) for pro in ep2_align.protein_list]
    mp1_charge_freqs = [analysis.frequency(pro, ['R', 'K']) for pro in mp1_align.protein_list]
    fish_charge_freqs = [analysis.frequency(pro, ['R', 'K']) for pro in fish_align.protein_list]
    # truncated eutherian P2
    ep2_trunc_freqs = analysis.region_frequencies(ep2_align, ['R', 'K'], 48)
    # DNA binding regions
    ep1_binding_freq = analysis.region_frequencies(ep1_align, ['R', 'K'], 17, 48)
    mp1_binding_freq = analysis.region_frequencies(mp1_align, ['R', 'K'], 17, 56)

    # Table 1-2: Low, quartile, and max values
    print("#################################")
    print("# Charged Residue Quartile Data #")
    print("#################################")
    print()

    table_1_freq_dict = {
        "Eutherian P1": ep1_charge_freqs,
        "Eutherian P2": ep2_charge_freqs,
        "Eutherian P2 T.": ep2_trunc_freqs,
        "Metatherian": mp1_charge_freqs,
        "Fish": fish_charge_freqs,
    }
    analysis.quartile_table(table_1_freq_dict)

    table_2_freq_dict = {"Eutherian P1 DNA": ep1_binding_freq, "Metatherian DNA": mp1_binding_freq}
    analysis.quartile_table(table_2_freq_dict)

    # t-Test Values
    print("###################################")
    print("# t-Tests for DNA Binding Regions #")
    print("###################################")
    print()
    print("ep1-mp1 ", '{:0.3e}'.format(ttest_ind(ep1_binding_freq, mp1_binding_freq, equal_var=False)[1]))
    print("ep1-fish", '{:0.3e}'.format(ttest_ind(ep1_binding_freq, fish_charge_freqs, equal_var=False)[1]))
    print("mp1-fish", '{:0.3e}'.format(ttest_ind(mp1_binding_freq, fish_charge_freqs, equal_var=False)[1]))
    print("\n")

    # variability analysis
    print("########################")
    print("# Variability Analysis #")
    print("########################")
    print()
    for x in range(5):
        print(alignment_names[x])
        if x == 2:
            analysis.variability_table(alignments[x], 48)
        else:
            analysis.variability_table(alignments[x])
