#!/usr/bin/python3
#
#   Written by Christian D. Powell, 10/14/2019
#   Copyright (c) 2019. Christian D. Powell and Hunter N.B. Moseley
#   All rights reserved.
#
from tempfile import NamedTemporaryFile
import subprocess


def align(fasta_str):
    """
    Method for taking a FASTA input and passing to MUSCLE to align.

    Creates a temporary text file to pass to MUSCLE standalone executable.

    :param fasta_str: FASTA format of protein sequences
    :type fasta_str: str
    :return: FASTA format of multiple sequence alignment of protein sequences.
    :rtype: str
    """
    with NamedTemporaryFile(mode='r+') as tmp_file:
        tmp_file.write(fasta_str)
        tmp_file.seek(0)

        muscle_cline = subprocess.Popen(['/usr/bin/muscle3.8.31_i86linux64', '-in', tmp_file.name,
                                         '-quiet'], cwd='/tmp/', stdout=subprocess.PIPE)
        aligned_fasta_str = muscle_cline.communicate()[0].decode("utf-8")

    return aligned_fasta_str
